@@ -156,10 +156,10 @@ static NTSTATUS smb2_create_blob_push_one(TALLOC_CTX *mem_ctx, DATA_BLOB *buffer
 NTSTATUS smb2_create_blob_push(TALLOC_CTX *mem_ctx, DATA_BLOB *buffer,
 			       const struct smb2_create_blobs blobs)
 {
-	int i;
+	uint32_t i;
 	NTSTATUS status;
 
-	*buffer = data_blob(NULL, 0);
+	*buffer = (DATA_BLOB) { 0 };
 	for (i=0; i < blobs.num_blobs; i++) {
 		bool last = false;
 		const struct smb2_create_blob *c;
