@@ -21,9 +21,7 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
-#if defined(__mips__) || defined(__mips64__) || defined(__sw_64__)
-#define _FILE_OFFSET_BITS 64
-#endif
+
 #include "includes.h"
 #include "libsmb/libsmb.h"
 #include "libsmbclient.h"
@@ -414,7 +412,7 @@ SMBC_fstatvfs_ctx(SMBCCTX *context,
                         /* ... then provide it */
                         st->f_bsize =
                                 (unsigned long) bytes_per_sector;
-#if HAVE_FRSIZE
+#ifdef HAVE_FRSIZE
                         st->f_frsize =
                                 (unsigned long) sectors_per_allocation_unit;
 #endif
@@ -422,6 +420,8 @@ SMBC_fstatvfs_ctx(SMBCCTX *context,
                                 (fsblkcnt_t) total_allocation_units;
                         st->f_bfree =
                                 (fsblkcnt_t) actual_allocation_units;
+                        st->f_bavail =
+                                (fsblkcnt_t) caller_allocation_units;
                 }
 
                 flags |= SMBC_VFS_FEATURE_NO_UNIXCIFS;
@@ -461,7 +461,7 @@ SMBC_fstatvfs_ctx(SMBCCTX *context,
                                 (fsfilcnt_t) total_file_nodes;
                         st->f_ffree =
                                 (fsfilcnt_t) free_file_nodes;
-#if HAVE_FSID_INT
+#ifdef HAVE_FSID_INT
                         st->f_fsid =
                                 (unsigned long) fs_identifier;
 #endif
@@ -491,9 +491,9 @@ SMBC_fstatvfs_ctx(SMBCCTX *context,
                 flags |= SMBC_VFS_FEATURE_DFS;
         }
 
-#if HAVE_STATVFS_F_FLAG
+#if defined(HAVE_STATVFS_F_FLAG)
         st->f_flag = flags;
-#elif HAVE_STATVFS_F_FLAGS
+#elif defined(HAVE_STATVFS_F_FLAGS)
         st->f_flags = flags;
 #endif
 
