@@ -57,16 +57,6 @@ bool winbindd_use_idmap_cache(void) {
 	return false;
 }
 
-bool idmap_is_online(void)
-{
-	return true;
-}
-
-NTSTATUS idmap_backends_unixid_to_sid(struct id_map *id)
-{
-	return NT_STATUS_OK;
-}
-
 static bool open_db(struct idmap_tdb_common_context *ctx)
 {
 	NTSTATUS status;
